

void initAll(){

   cam = new Capture(this, camW, camH);

  img = new PImage(80,60);
  bd = new BlobDetection(img.width, img.height);
 // bd.setPosDiscrimination(false);
  bd.setThreshold(0.2f);

  rks = new ReverseKeystone(this);
  //camSurface = rks.createReverseCornerPinSurface(320,240,5);
  camSurface = rks.createReverseCornerPinSurface( camW,camH,10);

ks = new Keystone(this);
projSurface = ks.createCornerPinSurface(width,height,5);

  camGraphics = createGraphics(camW,camH,P3D);
  projGraphics = createGraphics(width,height,JAVA2D);
  numPixels =  camW*camH;
  bgPixels = new int[numPixels];
frameRate(24);
  projGraphics.beginDraw();
  projGraphics.background(0);
  projGraphics.endDraw();
  
}

void trackDarkAreas(){
 bd.setPosDiscrimination(false);
}

void trackBrightAreas(){
 bd.setPosDiscrimination(true);
}

void readCamera(){

  if (cam.available() ) {
    cam.read();


    // The following does the same, and is faster when just drawing the image
    // without any additional resizing, transformations, or tint.
    //set(160, 100, cam);


  }
  //reverse keystone, getting this camera into the computer and make it rectangular
  camGraphics.beginDraw();
  camGraphics.image(cam, 0, 0);
  //draw the corner pin surface
  camSurface.render(camGraphics,cam);
  camGraphics.endDraw();

        backgroundSubtraction(camGraphics);
}

void computeBlobs(){


      img.copy(camGraphics, 0, 0, camGraphics.width, camGraphics.height, 
      0, 0, img.width, img.height);
      fastblur(img, 2);
      bd.computeBlobs(img.pixels);
}

void displayBlobs(){
    drawBlobsAndEdges(true,true,camGraphics.width, camGraphics.height);
}

void displayBlobs(PGraphics gr){
    drawBlobsAndEdges(gr,true,true,gr.width, gr.height);
}



void backgroundSubtraction(PGraphics g ){
  int presenceSum = 0;
  for (int i = 0; i < numPixels; i++) { // For each pixel in the video frame...
    // Fetch the current color in that location, and also the color
    // of the background in that spot
    color currColor = g.pixels[i];
    color bkgdColor = bgPixels[i];
    // Extract the red, green, and blue components of the current pixel’s color
    int currR = (currColor >> 16) & 0xFF;
    int currG = (currColor >> 8) & 0xFF;
    int currB = currColor & 0xFF;
    // Extract the red, green, and blue components of the background pixel’s color
    int bkgdR = (bkgdColor >> 16) & 0xFF;
    int bkgdG = (bkgdColor >> 8) & 0xFF;
    int bkgdB = bkgdColor & 0xFF;
    // Compute the difference of the red, green, and blue values
    int diffR = abs(currR - bkgdR);
    int diffG = abs(currG - bkgdG);
    int diffB = abs(currB - bkgdB);
    // Add these differences to the running tally
    presenceSum += diffR + diffG + diffB;
    // Render the difference image to the screen
    //pixels[i] = color(diffR, diffG, diffB);
    // The following line does the same thing much faster, but is more technical
    g.pixels[i] = 0xFF000000 | (diffR << 16) | (diffG << 8) | diffB;
  }
}




void drawBlobsAndEdges(boolean drawBlobs, boolean drawEdges, float w, float h)
{
  noFill();
  Blob b;
  EdgeVertex eA,eB;
  for (int n=0 ; n<bd.getBlobNb() ; n++)
  {
    b=bd.getBlob(n);
    if (b!=null)
    {
      // Edges
      if (drawEdges)
      {
        strokeWeight(3);
        stroke(0,255,0);
        for (int m=0;m<b.getEdgeNb();m++)
        {
          eA = b.getEdgeVertexA(m);
          eB = b.getEdgeVertexB(m);
          if (eA !=null && eB !=null)
            line(
            eA.x*w, eA.y*h, 
            eB.x*w, eB.y*h
              );
        }
      }

      // Blobs
      if (drawBlobs)
      {
        strokeWeight(1);
        stroke(255,0,0);
        rect(
        b.xMin*w,b.yMin*h,
        b.w*w,b.h*h
          );
      }

    }

  }
}



void drawBlobsAndEdges(PGraphics gr,boolean drawBlobs, boolean drawEdges, float w, float h)
{
  gr.beginDraw();
  gr.noFill();
  Blob b;
  EdgeVertex eA,eB;
  for (int n=0 ; n<bd.getBlobNb() ; n++)
  {
    b=bd.getBlob(n);
    if (b!=null)
    {
      // Edges
      if (drawEdges)
      {
        gr.strokeWeight(3);
        gr.stroke(0,255,0);
        for (int m=0;m<b.getEdgeNb();m++)
        {
          eA = b.getEdgeVertexA(m);
          eB = b.getEdgeVertexB(m);
          if (eA !=null && eB !=null)
            gr.line(
            eA.x*w, eA.y*h, 
            eB.x*w, eB.y*h
              );
        }
      }

      // Blobs
      if (drawBlobs)
      {
        gr.strokeWeight(1);
        gr.stroke(255,0,0);
        gr.rect(
        b.xMin*w,b.yMin*h,
        b.w*w,b.h*h
          );
      }

    }

  }
  
  gr.endDraw();
}


// ==================================================
// Super Fast Blur v1.1
// by Mario Klingemann 
// <http://incubator.quasimondo.com>
// ==================================================
void fastblur(PImage img,int radius)
{
  if (radius<1){
    return;
  }
  int w=img.width;
  int h=img.height;
  int wm=w-1;
  int hm=h-1;
  int wh=w*h;
  int div=radius+radius+1;
  int r[]=new int[wh];
  int g[]=new int[wh];
  int b[]=new int[wh];
  int rsum,gsum,bsum,x,y,i,p,p1,p2,yp,yi,yw;
  int vmin[] = new int[max(w,h)];
  int vmax[] = new int[max(w,h)];
  int[] pix=img.pixels;
  int dv[]=new int[256*div];
  for (i=0;i<256*div;i++){
    dv[i]=(i/div);
  }

  yw=yi=0;

  for (y=0;y<h;y++){
    rsum=gsum=bsum=0;
    for(i=-radius;i<=radius;i++){
      p=pix[yi+min(wm,max(i,0))];
      rsum+=(p & 0xff0000)>>16;
      gsum+=(p & 0x00ff00)>>8;
      bsum+= p & 0x0000ff;
    }
    for (x=0;x<w;x++){

      r[yi]=dv[rsum];
      g[yi]=dv[gsum];
      b[yi]=dv[bsum];

      if(y==0){
        vmin[x]=min(x+radius+1,wm);
        vmax[x]=max(x-radius,0);
      }
      p1=pix[yw+vmin[x]];
      p2=pix[yw+vmax[x]];

      rsum+=((p1 & 0xff0000)-(p2 & 0xff0000))>>16;
      gsum+=((p1 & 0x00ff00)-(p2 & 0x00ff00))>>8;
      bsum+= (p1 & 0x0000ff)-(p2 & 0x0000ff);
      yi++;
    }
    yw+=w;
  }

  for (x=0;x<w;x++){
    rsum=gsum=bsum=0;
    yp=-radius*w;
    for(i=-radius;i<=radius;i++){
      yi=max(0,yp)+x;
      rsum+=r[yi];
      gsum+=g[yi];
      bsum+=b[yi];
      yp+=w;
    }
    yi=x;
    for (y=0;y<h;y++){
      pix[yi]=0xff000000 | (dv[rsum]<<16) | (dv[gsum]<<8) | dv[bsum];
      if(x==0){
        vmin[y]=min(y+radius+1,hm)*w;
        vmax[y]=max(y-radius,0)*w;
      }
      p1=x+vmin[y];
      p2=x+vmax[y];

      rsum+=r[p1]-r[p2];
      gsum+=g[p1]-g[p2];
      bsum+=b[p1]-b[p2];

      yi+=w;
    }
  }

}
