import deadpixel.keystone.*;
import processing.video.*;
import deadpixel.revkeystone.*;
import blobDetection.*;

//********* CAMERA *****************//
//Reverse Keystone for the correction of perspective distortion in the camera image
ReverseKeystone rks;
ReverseCornerPinSurface camSurface;

//the Camera
Capture cam;

//camera dimensions
int camW = 320;
int camH = 240;

//BlobDetection detects objects in the camera image
BlobDetection bd;

//offscreen graphics where we store camera related stuff
PGraphics camGraphics;

//this PImage will be used for blob detection
PImage img;

//background pixel information, used for background subtraction
int [] bgPixels;
int numPixels;


//********* PROJECTION *****************//
//Keystone to correct perspective distortion in the projection
Keystone ks;
CornerPinSurface projSurface;

//offscreen graphics where we draw our scene into, this will be projected into real world
PGraphics projGraphics;


//*********** OTHER SETTINGS **************

//Display mode
int CAM = 0;
int PROJECTION = 1;
int mode = CAM;
boolean displayBlobs = true;




void setup(){
  size(1024,768,P3D);

  initAll();
 

}


void draw(){
  background(255);

  readCamera();
  
  //compute detection threshold based on mouse position
  float thres = map(mouseX,0,width,0,1);
  bd.setThreshold(thres);
  
  computeBlobs();

  //*********** START DRAW YOUR SCENE HERE *****************
  //example
  projGraphics.beginDraw();
  projGraphics.smooth();
  projGraphics.background(255);
  projGraphics.fill(0,0,255);
  projGraphics.stroke(0);
  for (int n=0 ; n<bd.getBlobNb() ; n++)
  {
    Blob  b=bd.getBlob(n);
    if (b!=null)
    {
      float x = map(b.x,0,1,0,projGraphics.width);
      float y = map(b.y,0,1,0,projGraphics.height);
      

  //draw an arrow
      float x1 = x;
      float y1 = y;
      float x2 = x1+100;
      float y2 = y1;
      
      projGraphics.pushMatrix();
      projGraphics.translate(-280,0);
      projGraphics.stroke(0);
      projGraphics.fill(0);
      projGraphics.strokeWeight(10);
      projGraphics.line(x1,y1,x2,y2);
      projGraphics.strokeWeight(1);
      projGraphics.noStroke();
      projGraphics.triangle(x2,y2,x2+30,y2,x2,y2+20);
      projGraphics.triangle(x2,y2,x2+30,y2,x2,y2-20);
      projGraphics.popMatrix();

    }

  }
  projGraphics.endDraw();

//****************** END DRAW YOUR SCENE HERE ******************

 //display the scene
  if(mode == CAM){
    //display camera image and reverse corner pin surface
    background(255);
    image(camGraphics,0,0);
    displayBlobs();

  }
  else if(mode == PROJECTION){
    //display white screen and corner pin surface
    //image(projGraphics,0,0);
    background(0);
    if(displayBlobs){
     displayBlobs(projGraphics);
    }
    projSurface.render(projGraphics);
   
  }
}


void keyPressed(){
  if(key =='c'){
    rks.toggleCalibration();
  }
  else if(key =='v'){
    ks.toggleCalibration();
  }
  else if(key == 'r'){
    camSurface.makeRectangular();
  }
  else if(key == 'b'){
    //store background for background subtraction
    println("learn background");
    camGraphics.loadPixels();
    arraycopy(camGraphics.pixels,bgPixels);
  }
  else if(key == 'm'){
    cam.settings();
  }
  else if(key == '1'){
    mode = CAM;
  }
  else if(key == '2'){
    mode = PROJECTION;
  }
  else if(key == '3'){
    displayBlobs = !displayBlobs;
  }
  else if(key == 's'){
    rks.save();
    ks.save();
  }
   else if(key == 'l'){
    rks.load();
    ks.load();
  }
  
}







